@@ -6,11 +6,11 @@ WWW::Mechanize - Handy web browsing in a Perl object
 
 =head1 VERSION
 
-Version 1.50
+Version 1.52
 
 =cut
 
-our $VERSION = '1.50';
+our $VERSION = '1.52';
 
 =head1 SYNOPSIS
 
@@ -431,11 +431,23 @@ The equivalent of hitting the "back" button in a browser.  Returns to
 the previous page.  Won't go back past the first page. (Really, what
 would it do if it could?)
 
+Returns true if it could go back, or false if not.
+
 =cut
 
 sub back {
     my $self = shift;
-    $self->_pop_page_stack;
+
+    my $stack = $self->{page_stack};
+    return unless $stack && @{$stack};
+
+    my $popped = pop @{$self->{page_stack}};
+    my $req    = $popped->{req};
+    my $res    = $popped->{res};
+
+    $self->_update_page( $req, $res );
+
+    return 1;
 }
 
 =head1 STATUS METHODS
@@ -469,9 +481,10 @@ Synonym for C<< $mech->response() >>
 
 =head2 $mech->status()
 
-Returns the HTTP status code of the response.
+Returns the HTTP status code of the response.  This is a 3-digit
+number like 200 for OK, 404 for not found, and so on.
 
-=head2 $mech->ct()
+=head2 $mech->ct() / $mech->content_type()
 
 Returns the content type of the response.
 
@@ -512,9 +525,10 @@ sub res {           my $self = shift; return $self->{res}; }
 sub response {      my $self = shift; return $self->{res}; }
 sub status {        my $self = shift; return $self->{status}; }
 sub ct {            my $self = shift; return $self->{ct}; }
+sub content_type {  my $self = shift; return $self->{ct}; }
 sub base {          my $self = shift; return $self->{base}; }
 sub current_form {  my $self = shift; return $self->{form}; }
-sub is_html {       my $self = shift; return defined $self->{ct} && ($self->{ct} eq 'text/html'); }
+sub is_html {       my $self = shift; return defined $self->ct && ($self->ct eq 'text/html'); }
 
 =head2 $mech->title()
 
@@ -690,6 +704,7 @@ sub follow_link {
 
     my $link = $self->find_link(%parms);
     return $self->get( $link->url ) if $link;
+    $self->die( 'Link not found: ', $link->url ) if $self->{autocheck};
     return;
 }
 
@@ -784,15 +799,17 @@ The links come from the following:
 
 =over 4
 
-=item C<< <A HREF=...> >>
+=item C<< <a href=...> >>
 
-=item C<< <AREA HREF=...> >>
+=item C<< <area href=...> >>
 
-=item C<< <FRAME SRC=...> >>
+=item C<< <frame src=...> >>
 
-=item C<< <IFRAME SRC=...> >>
+=item C<< <iframe src=...> >>
 
-=item C<< <META CONTENT=...> >>
+=item C<< <link href=...> >>
+
+=item C<< <meta content=...> >>
 
 =back
 
@@ -1148,6 +1165,10 @@ sub find_all_images {
 
 =head1 FORM METHODS
 
+These methods let you work with the forms on a page.  The idea is
+to choose a form that you'll later work with using the field methods
+below.
+
 =head2 $mech->forms
 
 Lists all the forms on the current page.  Each form is an L<HTML::Form>
@@ -1198,13 +1219,12 @@ Selects a form by name.  If there is more than one form on the page
 with that name, then the first one is used, and a warning is
 generated.
 
-If it is found, the form is returned as an L<HTML::Form> object and set internally
-for later use with Mech's form methods such as C<L</field()>> and C<L</click()>>.
+If it is found, the form is returned as an L<HTML::Form> object and
+set internally for later use with Mech's form methods such as
+C<L</field()>> and C<L</click()>>.
 
 Returns undef if no form is found.
 
-Note that this functionality requires libwww-perl 5.69 or higher.
-
 =cut
 
 sub form_name {
@@ -1223,6 +1243,36 @@ sub form_name {
     }
 }
 
+=head2 $mech->form_id( $name )
+
+Selects a form by ID.  If there is more than one form on the page
+with that ID, then the first one is used, and a warning is generated.
+
+If it is found, the form is returned as an L<HTML::Form> object and
+set internally for later use with Mech's form methods such as
+C<L</field()>> and C<L</click()>>.
+
+Returns undef if no form is found.
+
+=cut
+
+sub form_id {
+    my ($self, $formid) = @_;
+
+    my $temp;
+    my @matches = grep { defined($temp = $_->attr('id')) and ($temp eq $formid) } $self->forms;
+    if ( @matches ) {
+        $self->warn( 'There are ', scalar @matches, " forms with ID $formid.  The first one was used." )
+            if @matches > 1;
+        return $self->{form} = $matches[0];
+    }
+    else {
+        $self->warn( qq{ There is no form with ID "$formid"} );
+        return undef;
+    }
+}
+
+
 =head2 $mech->form_with_fields( @fields )
 
 Selects a form by passing in a list of field names it must contain.  If there
@@ -1262,6 +1312,9 @@ sub form_with_fields {
     }
 }
 
+=head1 FIELD METHODS
+
+These methods allow you to set the values of fields in a given form.
 
 =head2 $mech->field( $name, $value, $number )
 
@@ -1765,13 +1818,13 @@ sub submit_form {
 
     if ($args{'with_fields'}) {
         $fields || die q{must submit some 'fields' with with_fields};
-        $self->form_with_fields(keys %{$fields}) or die;
+        $self->form_with_fields(keys %{$fields}) or die "There is no form with the requested fields";
     }
     elsif ( my $form_number = $args{'form_number'} ) {
-        $self->form_number( $form_number ) or die;
+        $self->form_number( $form_number ) or die "There is no form numbered $form_number";
     }
     elsif ( my $form_name = $args{'form_name'} ) {
-        $self->form_name( $form_name ) or die;
+        $self->form_name( $form_name ) or die qq{There is no form named "$form_name"};
     }
     else {
         # No form selector was used.
@@ -1898,6 +1951,9 @@ sub stack_depth {
 Dumps the contents of C<< $mech->content >> into I<$filename>.
 I<$filename> will be overwritten.  Dies if there are any errors.
 
+If the content type does not begin with "text/", then the content
+is saved in binary mode.
+
 =cut
 
 sub save_content {
@@ -1905,6 +1961,7 @@ sub save_content {
     my $filename = shift;
 
     open( my $fh, '>', $filename ) or $self->die( "Unable to create $filename: $!" );
+    binmode $fh unless $self->content_type =~ m{^text/};
     print {$fh} $self->content or $self->die( "Unable to write to $filename: $!" );
     close $fh or $self->die( "Unable to close $filename: $!" );
 
@@ -1998,15 +2055,17 @@ sub dump_all {
 
 =head2 $mech->clone()
 
-Clone the mech object. We override here to be sure the cookie jar
-gets copied over
+Clone the mech object.  The clone will be using the same cookie jar
+as the original mech.
 
 =cut
 
 sub clone {
-    my $self = shift;
-    my $clone =  $self->SUPER::clone();
-    $clone->{cookie_jar} = $self->cookie_jar;
+    my $self  = shift;
+    my $clone = $self->SUPER::clone();
+
+    $clone->cookie_jar( $self->cookie_jar );
+
     return $clone;
 }
 
@@ -2122,36 +2181,52 @@ sub update_html {
 
 =head2 $mech->credentials( $username, $password )
 
-Provide credentials to be used for HTTP Basic authentication for all sites and
-realms until further notice.
+Provide credentials to be used for HTTP Basic authentication for
+all sites and realms until further notice.
 
-The four argument form described in L<LWP::UserAgent> is still supported.
+The four argument form described in L<LWP::UserAgent> is still
+supported.
 
 =cut
 
-{
-    my $saved_method;
+sub credentials {
+    my $self = shift;
 
-    sub credentials {
-        my $self = shift;
-        no warnings 'redefine'; ## no critic
-
-        if (@_ == 4) {
-            $saved_method
-                and *LWP::UserAgent::get_basic_credentials = $saved_method;
-            return $self->SUPER::credentials(@_);
-        }
-
-        @_ == 2
-            or $self->die( 'Invalid # of args for overridden credentials()' );
-
-        my ($username, $password) = @_;
-        $saved_method ||= \&LWP::UserAgent::get_basic_credentials;
-        *LWP::UserAgent::get_basic_credentials
-            = sub { return $username, $password };
+    # The lastest LWP::UserAgent also supports 2 arguments,
+    # in which case the first is host:port
+    if (@_ == 4 || (@_ == 2 && $_[0] =~ /:\d+$/)) {
+        return $self->SUPER::credentials(@_);
     }
+
+    @_ == 2
+        or $self->die( 'Invalid # of args for overridden credentials()' );
+
+    return @$self{qw( __username __password )} = @_;
 }
 
+=head2 $mech->get_basic_credentials( $realm, $uri, $isproxy )
+
+Returns the credentials for the realm and URI.
+
+=cut
+
+sub get_basic_credentials {
+    my $self = shift;
+    my @cred = grep { defined } @$self{qw( __username __password )};
+    return @cred if @cred == 2;
+    return $self->SUPER::get_basic_credentials(@_);
+}
+
+=head2 $mech->clear_credentials()
+
+Remove any credentials set up with C<credentials()>.
+
+=cut
+
+sub clear_credentials {
+    my $self = shift;
+    delete @$self{qw( __username __password )};
+}
 
 =head1 INTERNAL-ONLY METHODS
 
@@ -2313,11 +2388,12 @@ Resets the internal fields that track page parsed stuff.
 sub _reset_page {
     my $self = shift;
 
-    $self->{_extracted_links} = 0;
+    $self->{_extracted_links}  = 0;
     $self->{_extracted_images} = 0;
-    $self->{links} = [];
-    $self->{images} = [];
-    $self->{forms} = [];
+    $self->{links}             = [];
+    $self->{images}            = [];
+    $self->{forms}             = [];
+
     delete $self->{form};
 
     return;
@@ -2331,11 +2407,12 @@ property with L<WWW::Mechanize::Link> objects.
 =cut
 
 my %link_tags = (
-    a => 'href',
-    area => 'href',
-    frame => 'src',
+    a      => 'href',
+    area   => 'href',
+    frame  => 'src',
     iframe => 'src',
-    meta => 'content',
+    link   => 'href',
+    meta   => 'content',
 );
 
 sub _extract_links {
@@ -2358,7 +2435,7 @@ sub _extract_links {
 
 
 my %image_tags = (
-    img => 'src',
+    img   => 'src',
     input => 'src',
 );
 
@@ -2461,7 +2538,7 @@ sub _link_from_token {
         });
 } # _link_from_token
 
-=head2 $mech->_push_page_stack() / $mech->_pop_page_stack()
+=head2 $mech->_push_page_stack()
 
 The agent keeps a stack of visited pages, which it can pop when it needs
 to go BACK and so on.
@@ -2477,39 +2554,17 @@ object.
 sub _push_page_stack {
     my $self = shift;
 
+    my $req = $self->{req};
+    my $res = $self->{res};
+
+    return unless $req && $res && $self->stack_depth;
+
     # Don't push anything if it's a virgin object
-    if ( $self->{res} && $self->stack_depth ) {
-        my $save_stack = $self->{page_stack};
-        $self->{page_stack} = [];
-
-        my $clone = $self->clone;
-        push( @{$save_stack}, $clone );
-
-        while ( @{$save_stack} > $self->stack_depth ) {
-            shift @{$save_stack};
-        }
-        $self->{page_stack} = $save_stack;
-    }
-
-    return 1;
-}
-
-sub _pop_page_stack {
-    my $self = shift;
-
-    if ( $self->{page_stack} && @{$self->{page_stack}} ) {
-        my $popped = pop @{$self->{page_stack}};
-
-        # eliminate everything in self
-        foreach my $key ( keys %{$self} ) {
-            delete $self->{ $key }              unless $key eq 'page_stack';
-        }
-
-        # make self just like the popped object
-        foreach my $key ( keys %{$popped} ) {
-            $self->{ $key } = $popped->{ $key } unless $key eq 'page_stack';
-        }
+    my $stack = $self->{page_stack} ||= [];
+    if ( @{$stack} >= $self->stack_depth ) {
+        shift @{$stack};
     }
+    push( @{$stack}, { req => $req, res => $res } );
 
     return 1;
 }
@@ -2729,6 +2784,11 @@ Just like Mech, but using Microsoft Internet Explorer to do the work.
 Thanks to the numerous people who have helped out on WWW::Mechanize in
 one way or another, including
 Kirrily Robert for the original C<WWW::Automate>,
+Norbert Buchmuller,
+Dave Page,
+David Sainty,
+H.Merijn Brand,
+Matt Lawrence,
 Michael Schwern,
 Adriano Ferreira,
 Miyagawa,
